-----------------------------------------------------------------------------------------------
-- Client Lua Script for SimpleQuestTracker
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "QuestLib"
require "ChatSystemLib"

-----------------------------------------------------------------------------------------------
-- SimpleQuestTracker Module Definition
-----------------------------------------------------------------------------------------------
local SimpleQuestTracker = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local kstrTaskQuestMarker			= "Task"

local ktConToColor =
{
	[0] 												= "ffffffff",
	[Unit.CodeEnumLevelDifferentialAttribute.Grey] 		= "ff9aaea3",
	[Unit.CodeEnumLevelDifferentialAttribute.Green] 	= "ff37ff00",
	[Unit.CodeEnumLevelDifferentialAttribute.Cyan] 		= "ff46ffff",
	[Unit.CodeEnumLevelDifferentialAttribute.Blue] 		= "ff309afc",
	[Unit.CodeEnumLevelDifferentialAttribute.White] 	= "ffffffff",
	[Unit.CodeEnumLevelDifferentialAttribute.Yellow] 	= "ffffd400", -- Yellow
	[Unit.CodeEnumLevelDifferentialAttribute.Orange] 	= "ffff6a00", -- Orange
	[Unit.CodeEnumLevelDifferentialAttribute.Red] 		= "ffff0000", -- Red
	[Unit.CodeEnumLevelDifferentialAttribute.Magenta] 	= "fffb00ff", -- Purp
}

local ktConToString =
{
	[0] 												= Apollo.GetString("Unknown_Unit"),
	[Unit.CodeEnumLevelDifferentialAttribute.Grey] 		= Apollo.GetString("QuestLog_Trivial"),
	[Unit.CodeEnumLevelDifferentialAttribute.Green] 	= Apollo.GetString("QuestLog_Easy"),
	[Unit.CodeEnumLevelDifferentialAttribute.Cyan] 		= Apollo.GetString("QuestLog_Simple"),
	[Unit.CodeEnumLevelDifferentialAttribute.Blue] 		= Apollo.GetString("QuestLog_Standard"),
	[Unit.CodeEnumLevelDifferentialAttribute.White] 	= Apollo.GetString("QuestLog_Average"),
	[Unit.CodeEnumLevelDifferentialAttribute.Yellow] 	= Apollo.GetString("QuestLog_Moderate"),
	[Unit.CodeEnumLevelDifferentialAttribute.Orange] 	= Apollo.GetString("QuestLog_Tough"),
	[Unit.CodeEnumLevelDifferentialAttribute.Red] 		= Apollo.GetString("QuestLog_Hard"),
	[Unit.CodeEnumLevelDifferentialAttribute.Magenta] 	= Apollo.GetString("QuestLog_Impossible")
}

local kstrRed 		= "ffff4c4c"
local kstrGreen 	= "ff2fdc02"
local kstrYellow 	= "fffffc00"
local kstrLightGrey = "ffb4b4b4"
local kstrHighlight = "ffffe153"

-----------------------------------------------------------------------------------------------
-- Variables
-----------------------------------------------------------------------------------------------
local SimpleMode = true
local SimpleQuestTracker_loc = SimpleQuestTracker_loc_enUS
local CarbineQuestTracker = {}
local ResizeAll = nil
local CarbineDrawEpisode = nil
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SimpleQuestTracker:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function SimpleQuestTracker:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"QuestTracker",
		"ObjectiveTracker"
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)

	local strCancel = Apollo.GetString(1)
	-- German
	if strCancel == "Abbrechen" then 
		SimpleQuestTracker_loc = SimpleQuestTracker_loc_deDE
	end

	-- French
	if strCancel == "Annuler" then
		SimpleQuestTracker_loc = SimpleQuestTracker_loc_frFR
	end

	-- Other fall back on English
	-- Already set as default
end

--------------------------------------------------------------------------------------------
-- SimpleQuestTracker OnLoad
-----------------------------------------------------------------------------------------------
function SimpleQuestTracker:OnLoad()
	-- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SimpleQuestTracker.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SimpleQuestTracker OnDocLoaded
-----------------------------------------------------------------------------------------------
function SimpleQuestTracker:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "SimpleQuestTrackerForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end

		self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		self.xmlDoc = nil

		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("sqt", "OnSlashCommand", self)

		-- Do additional Addon initialization here
		CarbineQuestTracker = Apollo.GetAddon("QuestTracker")
		CarbineBuildAll = CarbineQuestTracker["BuildAll"]
		CarbineDrawQuest = CarbineQuestTracker["DrawQuest"]
		assert(CarbineBuildAll, "Carbine original QuestTracker:BuildAll function not found")
		assert(CarbineDrawQuest, "Carbine original QuestTracker:DrawQuest function not found")
		
		self:SetMode()
	end
end

function SimpleQuestTracker:OnSave(eType)
	if eType == GameLib.CodeEnumAddonSaveLevel.Account then 
		local tSave = {}
		tSave.SimpleMode = SimpleMode

		return tSave
	end

	return nil
end

function SimpleQuestTracker:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Account then 
		SimpleMode = tSavedData.SimpleMode
	end
end

-----------------------------------------------------------------------------------------------
-- SimpleQuestTracker Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- Handle the add-ons only /command to set the quest tracker mode
function SimpleQuestTracker:OnSlashCommand(cmd, args)
	SimpleMode = not SimpleMode
	self:SetMode()
end

-- Set the quest tracker to use the specified behavior
function SimpleQuestTracker:SetMode()
	self:DestroyAllEpisodes()

	if SimpleMode then
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "[SimpleQuestTracker]: Enabled. Type /sqt to toggle SimpleQuestTracker.")

		CarbineQuestTracker["BuildAll"]=self["BuildAll"]
		CarbineQuestTracker["DrawQuest"]=self["DrawQuest"]
	else
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "[SimpleQuestTracker]: Disabled. Type /sqt to toggle SimpleQuestTracker.")

		CarbineQuestTracker["BuildAll"]=CarbineBuildAll
		CarbineQuestTracker["DrawQuest"]=CarbineDrawQuest
	end
	
	CarbineQuestTracker:BuildAll()
end

-- Destroy all the existing episodes in the QuestTracker
function SimpleQuestTracker:DestroyAllEpisodes()
	for idx, epiEpisode in pairs(QuestLib.GetTrackedEpisodes(self.bQuestTrackerByDistance)) do
		CarbineQuestTracker:DestroyEpisode(epiEpisode)
	end
end

-- Replacement function to build quest tracker with only a single Episode bucket
function SimpleQuestTracker:BuildAll()
	if not CarbineQuestTracker.bQuestsInitialized then
		return
	end
	
	for idx, epiEpisode in pairs(QuestLib.GetTrackedEpisodes(self.bQuestTrackerByDistance)) do
		for nIdx, queQuest in pairs(epiEpisode:GetTrackedQuests(0, self.bQuestTrackerByDistance)) do
			CarbineQuestTracker:DrawQuest(queQuest)
		end
	end
	CarbineQuestTracker:BuildContentFinderNotification()
	CarbineQuestTracker:ResizeContentFinderNotification()
end

-- Replacement function to draw all quests under a single Episoide bucket
function SimpleQuestTracker:DrawQuest(queQuest)
	if not queQuest:IsTracked() then
		CarbineQuestTracker:DestroyQuest(queQuest)
		return
	end

	local tQuest = CarbineQuestTracker.tQuestWndCache[queQuest:GetId()]
	local wndQuest = nil
	if tQuest ~= nil then
		wndQuest = tQuest.wndQuest
	end
	if wndQuest == nil or not wndQuest:IsValid() then
		local epiEpisode = queQuest:GetEpisode()
		if epiEpisode == nil then
			return
		end
		local wndContainer = nil
	
		if CarbineQuestTracker.tPinned.tQuests[queQuest:GetId()] then
			local wndGroup = CarbineQuestTracker.tGroupWndCache[kstrPinnedQuestMarker]
			if wndGroup == nil or not wndGroup:IsValid() then
				return
			end
			wndContainer = wndGroup:FindChild("EpisodeGroupContainer")
		elseif queQuest:IsImbuementQuest() then
			local wndGroup = CarbineQuestTracker.tGroupWndCache[kstrImbuementQuestMarker]
			if wndGroup == nil or not wndGroup:IsValid() then
				return
			end
			wndContainer = wndGroup:FindChild("EpisodeGroupContainer")
		else
			local wndGroup = CarbineQuestTracker.tGroupWndCache[kstrTaskQuestMarker]
			if wndGroup == nil or not wndGroup:IsValid() then
				return
			end
			wndContainer = wndGroup:FindChild("EpisodeGroupContainer")
		end
		
		wndQuest = Apollo.LoadForm(CarbineQuestTracker.xmlDoc, "QuestItem", wndContainer, CarbineQuestTracker)
		wndQuest:SetData({ queQuest = queQuest, nSort = 99 })
		CarbineQuestTracker.tQuestWndCache[queQuest:GetId()] = { wndQuest = wndQuest, queQuest = queQuest, epiEpisode = epiEpisode }
	end
	
	-- Quest Title
	local strTitle = queQuest:IsBreadcrumb() and Apollo.GetString("QuestTracker_Intro") .. queQuest:GetTitle() or queQuest:GetTitle()
	local eQuestState = queQuest:GetState()
	if eQuestState == Quest.QuestState_Botched then
		strTitle = string.format("<T Font=\"CRB_Interface10\" TextColor=\"%s\">%s</T>", kstrRed, String_GetWeaselString(Apollo.GetString("QuestTracker_Failed"), strTitle))
	elseif eQuestState == Quest.QuestState_Achieved then
		strTitle = string.format("<T Font=\"CRB_Interface10\" TextColor=\"%s\">%s</T>", kstrGreen,String_GetWeaselString(Apollo.GetString("QuestTracker_Complete"), strTitle))
	elseif (eQuestState == Quest.QuestState_Accepted or eQuestState == Quest.QuestState_Achieved) and queQuest:IsQuestTimed() then
		strTitle = CarbineQuestTracker:HelperBuildTimedQuestTitle(queQuest)
		CarbineQuestTracker.tTimedQuests[queQuest:GetId()] = { queQuest = queQuest, wndTitleFrame = wndQuest:FindChild("TitleText") }
		CarbineQuestTracker.timerRealTimeUpdate:Start()
	else
		local strColor = CarbineQuestTracker.tActiveProgBarQuests[queQuest:GetId()] and "ffffffff" or kstrLightGrey
		local crLevelConDiff = ktConToColor[queQuest:GetColoredDifficulty() or 0]
		strTitle = string.format("<T Font=\"CRB_Interface10\" TextColor=\"%s\">%s </T><T Font=\"CRB_Interface10\" TextColor=\"%s\">(%s)</T>", "ffffffff", strTitle, crLevelConDiff, queQuest:GetConLevel())

	end
	
	wndQuest:FindChild("TitleText"):SetAML(strTitle)
	wndQuest:FindChild("TitleText"):SetHeightToContentHeight()

	wndQuest:FindChild("QuestOpenMapBtn"):SetData(queQuest)
	wndQuest:FindChild("QuestCallbackBtn"):SetData(queQuest)
	wndQuest:FindChild("QuestGearBtn"):SetData(queQuest)
	wndQuest:FindChild("ControlBackerBtn"):SetData({ queQuest = queQuest, wndQuest =  wndQuest })
	wndQuest:FindChild("ControlBackerBtn"):SetText(GameLib.GetKeyBinding("Interact").." >")
	
	CarbineQuestTracker:HelperSelectInteractHintArrowObject(queQuest, wndQuest:FindChild("ControlBackerBtn"))
	
	local bMinimized = CarbineQuestTracker.tMinimized.tQuests[queQuest:GetId()]
	local wndQuestNumber = wndQuest:FindChild("QuestNumber")
	local wndQuestNumberBackerArt = wndQuest:FindChild("QuestNumberBackerArt")
	local wndObjectiveContainer = wndQuest:FindChild("ObjectiveContainer")

	-- Conditional drawing
	wndQuest:FindChild("QuestNumberUpdateHighlight"):Show(CarbineQuestTracker.tActiveProgBarQuests[queQuest:GetId()] ~= nil)
	wndQuestNumber:SetText(nIdx)
	wndQuestNumber:SetTextColor(ApolloColor.new("ff31fcf6"))
	wndQuest:FindChild("QuestCompletedBacker"):Show(false)
	wndQuestNumberBackerArt:SetBGColor(CColor.new(1,1,1,1))
	wndQuestNumberBackerArt:SetSprite("sprQT_NumBackerNormal")
	
	-- State depending drawing
	if eQuestState == Quest.QuestState_Botched then
		CarbineQuestTracker:HelperShowQuestCallbackBtn(wndQuest, queQuest, "sprQT_NumBackerFailed", "CRB_QuestTrackerSprites:btnQT_QuestFailed")

	elseif eQuestState == Quest.QuestState_Achieved then
		CarbineQuestTracker:HelperShowQuestCallbackBtn(wndQuest, queQuest, "sprQT_NumBackerCompleted", "CRB_QuestTrackerSprites:btnQT_QuestRedeem")

		for idx, tObjective in pairs(queQuest:GetVisibleObjectiveData()) do
			if tObjective.nCompleted >= tObjective.nNeeded then
				CarbineQuestTracker:DestroyObjective(queQuest, tObjective.nIndex)
			end
		end
		CarbineQuestTracker:DrawQuestSpell(queQuest, wndQuest)

		local wndObjective = wndObjectiveContainer:FindChild("ObjectiveCompleted")
		if wndObjective == nil or not wndObjective:IsValid() then
			wndObjective = Apollo.LoadForm(CarbineQuestTracker.xmlDoc, "QuestObjectiveItem", wndQuest:FindChild("ObjectiveContainer"), CarbineQuestTracker)
			wndObjective:SetName("ObjectiveCompleted")
		end
		
		wndObjective:FindChild("QuestObjectiveBtn"):SetData({["queOwner"] = queQuest, ["nObjectiveIdx"] = nil})
		wndObjective:FindChild("QuestObjectiveText"):SetAML(self:BuildObjectiveTitleString(queQuest))

	else
		CarbineQuestTracker:DrawQuestSpell(queQuest, wndQuest)
		
		for idx, tObjective in pairs(queQuest:GetVisibleObjectiveData()) do
			if tObjective.nCompleted < tObjective.nNeeded then
				CarbineQuestTracker:DrawObjective(queQuest, tObjective.nIndex)
			else
				CarbineQuestTracker:DestroyObjective(queQuest, tObjective.nIndex)
			end
		end
	end
	
end

-----------------------------------------------------------------------------------------------
-- SimpleQuestTracker Instance
-----------------------------------------------------------------------------------------------
local SimpleQuestTrackerInst = SimpleQuestTracker:new()
SimpleQuestTrackerInst:Init()